function H = smhank(varargin)
%SMHANK Toeplitz matrix class constructor.
%
%   H = SMHANK creates a default empty object.
%
%   H = SMHANK(col,lrow) creates a smhank object from the vector 
%       col containing the first column and the vector lrow
%       containing the last row of a Hankel matrix.
%
%   H = SMHANK(h,m,n) creates a smhank object from the vector h
%       of dimension (m+n-1) containing a (m-by-n) Hankel
%       matrix in HPS (Hankel Packed Storage) format.
%
%   See also SMTOEP/TOEPREM, SMCIRC/SMTOEP.

%Input Parameters
%    col	the first column of the Toeplitz matrix (full or sparse)
%    row  	the first row of the Toeplitz matrix (full or sparse)
%    t		Toeplitz matrix in TPS format (full or sparse)
%    m  	number of rows of the Toeplitz matrix
%    n  	number of columns of the Toeplitz matrix
%
%Description of the structure array (field names)
%    t          vector containing a Toeplitz matrix in TPS format
%    dim1       number of rows
%    dim2       number of columns
%    cev	eigenvalues of a related circulant matrix needed
%               for matrix-vector multiplication with the Toeplitz
%               matrix (this field is filled by the function TOEPREM)
%
%  Giuseppe Rodriguez, University of Cagliari, Italy
%       Email: rodriguez@unica.it
%
%  Last revised May 22, 2011

if nargin > 3
	error ('Too many input arguments.')
end
if (nargin == 0) || isempty(varargin{1})
	% If no input arguments, or empty vector, create a default object.
	H.h = [];
	H.dim1 = 0;
	H.dim2 = 0;
	H.cev = [];			
	H = class(H,'smhank');
elseif isa(varargin{1},'smhank')
	% If argument varargin{1} of class smhank, simply return it.
	H = varargin{1};
elseif ~isfloat(varargin{1})
	% If argument varargin{1} is not a floating point array, then stop.
	error('The type of the first argument is not correct.')
else
	switch nargin
	case 1
	% Check for the correct types of arguments.
		if min(size(varargin{1})) ~= 1 
			error('The first argument is not a vector.')
		end
	% If single argument col, uses this vector as first column
	% of a Hankel matrix.
		varargin{1} = varargin{1}(:);
		dim1 = size(varargin{1},1);
		H.h = [varargin{1};zeros(dim1-1,1)];
		H.dim1 = dim1;
		H.dim2 = dim1;
		H.cev = [];
	case 2
	% Check for the correct types of arguments.
		if (min(size(varargin{1})) ~= 1) || ...
		   (min(size(varargin{2})) ~= 1)
			error('The first or second argument is not a vector.')
		elseif ~isfloat(varargin{2})
		% If argument varargin{2} is not a floating point array, then
		% stop.
			error('The type of the second argument is not correct.')
		end
	% Create a smhank object for a Toeplitz matrix having first column
	% varargin{1} and first row varargin{2}.
		varargin{1} = varargin{1}(:);
		varargin{2} = varargin{2}(:);
	% col and row must have the same first component.
		if varargin{1}(end) ~= varargin{2}(1)
			warning('SMT:smhank:DiagonalConflict',['Last ' ...
				'element of input column does not ' ...
				'match first element of input ' ...
				'\nrow. Column wins diagonal conflict.'])
		end
		dim1 = size(varargin{1},1);
		dim2 = size(varargin{2},1);
		H.h = [varargin{1};varargin{2}(2:dim2)];
		H.dim1 = dim1;
		H.dim2 = dim2;
		H.cev = [];
	case 3
	% Check for the correct types of arguments.
		if min(size(varargin{1})) ~= 1
			error('The first argument is not a vector.')
		elseif ~isnumeric(varargin{2}) || ...
		   ~isnumeric(varargin{3}) || ...
		   (max(size(varargin{2})) ~= 1) || ...
		   (max(size(varargin{3})) ~= 1)
			error('The second or third argument is not a number.')
		end
	% Create a smhank object for a (m-by-n) Toeplitz matrix stored
	% in TPS format.
		varargin{1} = varargin{1}(:);
		if (size(varargin{1},1) ~= (varargin{2}+varargin{3}-1)) || ...
		   any([varargin{2} varargin{3}]<=0)
			error('Dimensions do not agree.')
		end
		H.h = varargin{1};
		H.dim1 = varargin{2};
		H.dim2 = varargin{3};
		H.cev = [];
	end
	H = class(H,'smhank');
end
if smtconfig('tpautoeig')
	H = hkprem(H);
end

